#/*
# * +--------------------------------------------------------------------------+
# * | Copyright 2002 Rational Software Corporation.  All Rights Reserved.      |
# * | This software contains proprietary and confidential information of       |
# * | Rational and its suppliers.  Use, disclosure or reproduction is          |
# * | prohibited without the prior express written consent of Rational.        |
# * +--------------------------------------------------------------------------+
# */
package PjCLib;
####################################################################################
# Module: PjCLib
#
# Description:
#    Contains useful subroutines you can use in your ProjectConsole transformations.
#    NOTE: Always call initalizeTables first before using any of the other exported
#    subroutines.
#
# Exported Subroutines:
#     initializeTables
#     getPhysicalTableDbName
#     getPhysicalFieldDbName
#     getCQTableDbName
#     getCQFieldDbName
#
#     dumpPhysicalTableNames
#     dumpPhysicalFieldNames
#     dumpCQTableNames
#     dumpCQFieldNames
#
#     getPhysicalReferenceValue
#     getPhysicalReferenceDBID
#
# Exported Variables:
#     BOOLEAN_TRUE
#     BOOLEAN_FALSE
# 
####################################################################################

use Win32::OLE;
#require CQPerlExt;

@ISA = qw(Exporter DynaLoader);

# The following subroutines are externally visible
@EXPORT = qw(initializeTables, getPhysicalTableDbName, getPhysicalFieldDbName, 
             getCQTableDbName, getCQFieldDbName,
             dumpPhysicalTableNames, dumpPhysicalFieldNames,
             dumpCQTableNames, dumpCQFieldNames,
             getPhysicalReferenceValue, getPhysicalReferenceDBID);

# The following constants are externally visible
@EXPORT_OK = (BOOLEAN_TRUE, BOOLEAN_FALSE);

use constant BOOLEAN_TRUE => 1;
use constant BOOLEAN_FALSE => 0;

#######################################################
# These are private variables:

# Keep track of whether the hash tables are loaded.
$hashTablesLoaded = BOOLEAN_FALSE;
# Hash tables for looking up table names

# Hash table for the entitydef table
$entityDefHash = ("");
# Hash table for the PjC dbm_table table
$pjcTableHash = ("");

# Hash table for the fielddef table
$fieldDefHash = ("");
# Hash table for the PjC dbm_field table
$pjcFieldHashKey = ("");
$pjcFieldHash = ("");

#######################################################



#
# Name: initializePjCLibrary
#
# Description:
#    This subroutine initializes the hash tables used for looking up table names
#    and field names.  Values are read from the ProjectConsole metric warehouse.
#    
#    This subroutine MUST be called first in to utilize the exported subroutines.
#    You can call this again and again if you wish to refresh the hash table with
#    data from the metric warehouse.
#
# Input Parameters:
#    $currentSession - The ClearQuest session variable.  This *must* be an active
#        ClearQuest session.
#    $reload         - Pass in PjCLib->BOOLEAN_TRUE or PjCLib->BOOLEAN_FALSE to
#        tell this module to refresh its hash tables.
#
sub initializePjCLibrary()
{
    my ($parentPkgName, $currentSession, $reload) = @_;
    # NOTE: You do not need to pass in the first parameter $parentPkgName. The value
    # of this parameter is always "PjCLib" and is set by Perl on externally visible
    # subroutines. 

    $session = $currentSession;

    if ($reload == BOOLEAN_TRUE)
    {
        $hashTablesLoaded = BOOLEAN_FALSE;
    }
    loadEntityDefTableNames();

    loadPjCTableNames();

    loadFieldDefNames();

    loadPjCFieldNames();

    $hashTablesLoaded = BOOLEAN_TRUE;
}

#
# Name: getPhysicalTableDbName
#
# Description:
#    This subroutine returns the physical database table name for the specified
#    ProjectConsole target table display name.
#
# Input Parameters:
#    $displayName - The ProjectConsole table display name.  This is name you have
#        specified in the Dashboard Designer when you created the target table.
#        Value is case-sensitive.
#
# Return Value: 
#    String containing the physical table name.  If the value returned is "", then the 
#    table was not found.  Make sure the parameters are valid target table 
#    display names.  Parameter values are also case-sensitive.
#     
sub getPhysicalTableDbName()
{
    my ($parentPkgName, $displayName) = @_;
    # NOTE: You do not need to pass in the first parameter $parentPkgName. The value
    # of this parameter is always "PjCLib" and is set by Perl. 

    return $entityDefHash{$pjcTableHash{$displayName}}
}

#
# Name: getCQTableDbName
#
# Description:
#    This subroutine returns the ClearQuest database table name for the specified
#    ProjectConsole target table display name.
#
# Input Parameters:
#    $displayName - The ProjectConsole table display name.  This is name you have
#        specified in the Dashboard Designer when you created the target table.
#        Value is case-sensitive.
#
# Return Value: 
#    String containing the ClearQuest table name.  If the value returned is "", then the 
#    table was not found.  Make sure the parameters are valid target table 
#    display names.  Parameter values are also case-sensitive.
#     
sub getCQTableDbName()
{
    my ($parentPkgName, $displayName) = @_;
    # NOTE: You do not need to pass in the first parameter $parentPkgName. The value
    # of this parameter is always "PjCLib" and is set by Perl. 

    return $pjcTableHash{$displayName}
}

#
# Name: getPhysicalFieldDbName
#
# Description:
#    This subroutine returns the physical database field name for the specified
#    ProjectConsole target table field display name.
#
# Input Parameters:
#    $displayTableName - The ProjectConsole table display name.  This is name you have
#        specified in the Dashboard Designer when you created the target table.  Value
#        is case-sensitive.
#    $displayFieldName - The ProjectConsole field display name.  This is name you have
#        specified in the Dashboard Designer when you created the field in the specified
#        target table.  Value is case-sensitive.
#
# Return Value: 
#    String containing the physical field name.  If the value returned is "", then the 
#    field was not found.  Make sure the parameters are valid target table and field 
#    display names.  Input parameter values are also case-sensitive.
#
sub getPhysicalFieldDbName()
{
    my ($parentPkgName, $displayTableName, $displayFieldName) = @_;
    # NOTE: You do not need to pass in the first parameter $parentPkgName. The value
    # of this parameter is always "PjCLib" and is set by Perl. 

    return $fieldDefHash{$pjcFieldHashKey{$displayTableName . "." . $displayFieldName}}
}

#
# Name: getCQFieldDbName
#
# Description:
#    This subroutine returns the ClearQuest database field name for the specified
#    ProjectConsole target table field display name.
#
# Input Parameters:
#    $displayTableName - The ProjectConsole table display name.  This is name you have
#        specified in the Dashboard Designer when you created the target table.  Value
#        is case-sensitive.
#    $displayFieldName - The ProjectConsole field display name.  This is name you have
#        specified in the Dashboard Designer when you created the field in the specified
#        target table.  Value is case-sensitive.
#
# Return Value: 
#    String containing the ClearQuest field name.  If the value returned is "", then the 
#    field was not found.  Make sure the parameters are valid target table and field 
#    display names.  Input parameter values are also case-sensitive.
#
sub getCQFieldDbName()
{
    my ($parentPkgName, $displayTableName, $displayFieldName) = @_;
    # NOTE: You do not need to pass in the first parameter $parentPkgName. The value
    # of this parameter is always "PjCLib" and is set by Perl. 

    return $pjcFieldHash{$displayTableName . "." . $displayFieldName}

}


#
# Name: dumpPhysicalTableNames
#
# Description:
#    This subroutine prints out the translation table of ProjectConsole table display names
#    (the names you enter in the Designer) and the actual physical database table names.
#    Output is to both standard out and to a file, PhysicalTableNames.txt.
#
# Input Parameters: None
#
# Return Value: None
#
sub dumpPhysicalTableNames {
format PHYSICALTABLE =
@<<<<<<<<<<<<<<<<<<<<<<<<< @<<<<<<<<<<<<<<<<<<<<<<<<<
$defaultDisplayName,       $physicalName
.

    print "\n\nTranslation of ProjectConsole target table display names to physical table names:\n";
    print "(Output also saved in file PhysicalTableNames.txt.)\n\n";
    print "Display Name              Physical Table Name\n";
    print "------------------------- -----------------------\n";

    open (PHYSICALTABLE, ">PhysicalTableNames.txt");
    print PHYSICALTABLE "Translation of ProjectConsole target table display names to physical table names:\n\n";
    print PHYSICALTABLE "Display Name              Physical Table Name\n";
    print PHYSICALTABLE "------------------------- -----------------------\n";

    $queryStr = "SELECT dbm_table.default_display_name_1,entitydef.db_name " .
                "FROM dbm_table,entitydef " .
                "WHERE dbm_table.name=entitydef.name " .
                "ORDER BY dbm_table.default_display_name_1";
    #print "Excute query $queryStr...\n";

    my $resultSetObj = $session->BuildSQLQuery($queryStr);
    $resultSetObj->Execute();
    while ($resultSetObj->MoveNext() == $CQPerlExt::CQ_SUCCESS) {
       $defaultDisplayName = $resultSetObj->GetColumnValue(1);
       $physicalName = $resultSetObj->GetColumnValue(2);
       $physicalName =~ tr/A-Z/a-z/;  # CQ lowercases all names
       printf "%-25s %-25s\n", $defaultDisplayName, $physicalName;
       write (PHYSICALTABLE);
    }


    #foreach $key (keys %pjcTableHash)
    #{
    #   #print "$key		$entityDefHash{$pjcTableHash{$key}}\n";
    #   printf "%-25s %-25s\n", $key, $entityDefHash{$pjcTableHash{$key}};
    #   $physicalname = $entityDefHash{$pjcTableHash{$key}};
    #   write (PHYSICALTABLE);
    #}

    close (PHYSICALTABLE);
    print "\nOutput also saved in file PhysicalTableNames.txt.\n\n";
}

sub dumpPhysicalFieldNames {

format OUTPUTFILE =
@<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< @<<<<<<<<<<<<<<<<<<<<<<<<<
$displayName                                       $cqFieldName
.

    print "\n\nTranslation of ProjectConsole target table field display names to physical field names:\n";
    print "(Output also saved in file PhysicalFieldNames.txt.)\n\n";
    print "ProjectConsole Display Name                        Physical Field Name\n";
    print "-------------------------------------------------- -----------------------\n";

    open (OUTPUTFILE, ">PhysicalFieldNames.txt");
    print OUTPUTFILE "Translation of ProjectConsole target table field display names to physical field names:\n\n";
    print OUTPUTFILE "ProjectConsole Display Name                        Physical Field Name\n";
    print OUTPUTFILE "-------------------------------------------------- -----------------------\n";

    ######################################################################################################
    # This query doesn't quite return the right result set.  For some reason, there are duplicates in
    # the result set.
    #$queryStr = "SELECT dbm_table.default_display_name_1,dbm_field.default_display_name_1,fielddef.db_name " .
    #            "FROM dbm_table,dbm_field,fielddef,entitydef " .
    #            "WHERE (dbm_field.name=fielddef.name and dbm_field.table_fk=dbm_table.dbid " .
    #            "and dbm_table.name= entitydef.name) " .
    #            "ORDER BY dbm_table.default_display_name_1";
    #
    # So, to get the right result set, just query for the cq names, which should be the same as the
    # the physical names except all physical names are lower case.  So, we will have to convert the
    # results to lower case.
    $queryStr = "SELECT dbm_table.default_display_name_1,dbm_field.default_display_name_1,dbm_field.name " .
                "FROM dbm_table,dbm_field " .
                "WHERE dbm_table.dbid=dbm_field.table_fk " .
                "ORDER BY dbm_table.default_display_name_1";
    #print "Excute query $queryStr...\n";

    my $resultSetObj = $session->BuildSQLQuery($queryStr);
    $resultSetObj->Execute();
    while ($resultSetObj->MoveNext() == $CQPerlExt::CQ_SUCCESS) {
       $defaultTableDisplayName = $resultSetObj->GetColumnValue(1);
       $defaultFieldDisplayName = $resultSetObj->GetColumnValue(2);
       $cqFieldName = $resultSetObj->GetColumnValue(3);
       $cqFieldName =~ tr/A-Z/a-z/;  # CQ lowercases all names
       $displayName = $defaultTableDisplayName . "." . $defaultFieldDisplayName;
       printf "%-50s %-25s\n", $displayName, $cqFieldName;
       write (OUTPUTFILE);
    }

    #foreach $key (keys %pjcFieldHashKey)
    #{
    #   #print "$key		$fieldDefHash{$pjcFieldHashKey{$key}}\n";
    #   printf "%-50s %-25s\n", $key, $fieldDefHash{$pjcFieldHashKey{$key}};
    #   $physicalname = $fieldDefHash{$pjcFieldHashKey{$key}};
    #   write (OUTPUTFILE);
    #}
    close (OUTPUTFILE);

    print "\nOutput also saved in file PhysicalFieldNames.txt.\n\n";
}

#
# Name: dumpCQTableNames
#
# Description:
#    This subroutine prints out the translation table of ProjectConsole table display names
#    (the names you enter in the Designer) and the ClearQuest table names.
#    Output is to both standard out and to a file, CQTableNames.txt.
#
# Input Parameters: None
#
# Return Value: None
#
sub dumpCQTableNames {
format CQTABLE =
@<<<<<<<<<<<<<<<<<<<<<<<<< @<<<<<<<<<<<<<<<<<<<<<<<<<
$defaultDisplayName,       $cqName
.

    print "\n\nTranslation of ProjectConsole target table display names to CQ table names:\n";
    print "(Output also saved in file CQTableNames.txt.)\n\n";
    print "Display Name              CQ Table Name\n";
    print "------------------------- ------------------------\n";

    open (CQTABLE, ">CQTableNames.txt");
    print CQTABLE "Translation of ProjectConsole target table display names to CQ table names:\n\n";
    print CQTABLE "Display Name              CQ Table Name\n";
    print CQTABLE "------------------------- ------------------------\n";

    $queryStr = "SELECT  dbm_table.default_display_name_1, dbm_table.name" .
                " FROM dbm_table" . 
                " ORDER BY dbm_table.default_display_name_1";
    #print "Excute query $queryStr...\n";

    my $resultSetObj = $session->BuildSQLQuery($queryStr);
    $resultSetObj->Execute();
    while ($resultSetObj->MoveNext() == $CQPerlExt::CQ_SUCCESS) {
       $defaultDisplayName = $resultSetObj->GetColumnValue(1);
       $cqName = $resultSetObj->GetColumnValue(2);
       $cqName =~ tr/A-Z/a-z/;  # CQ lowercases all names
       printf "%-25s %-25s\n", $defaultDisplayName, $cqName;
       write (CQTABLE);
    }


    #foreach $key (keys %pjcTableHash)
    #{
    #   #printf "$key		$pjcTableHash{$key}\n";
    #   printf "%-25s %-25s\n", $key, $pjcTableHash{$key};
    #   $cqname = $pjcTableHash{$key};
    #   write (CQTABLE);
    #}

    close (CQTABLE);
    print "\nOutput also saved in file CQTableNames.txt.\n\n";
}

sub dumpCQFieldNames {
format OUTPUTFILE =
@<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< @<<<<<<<<<<<<<<<<<<<<<<<<<
$displayName                                       $cqFieldName
.

    print "\n\nTranslation of ProjectConsole target table field display names to ClearQuest field names:\n";
    print "(Output also saved in file CQFieldNames.txt.)\n\n";
    print "ProjectConsole Display Name                        CQ Field Name\n";
    print "-------------------------------------------------- -----------------------\n";

    open (OUTPUTFILE, ">CQFieldNames.txt");
    print OUTPUTFILE "Translation of ProjectConsole target table field display names to ClearQuest field names:\n\n";
    print OUTPUTFILE "ProjectConsole Display Name                        CQ Field Name\n";
    print OUTPUTFILE "-------------------------------------------------- -----------------------\n";

    $queryStr = "SELECT dbm_table.default_display_name_1,dbm_field.default_display_name_1,dbm_field.name " .
                "FROM dbm_table,dbm_field " .
                "WHERE dbm_table.dbid=dbm_field.table_fk " .
                "ORDER BY dbm_table.default_display_name_1";
    #print "Excute query $queryStr...\n";

    my $resultSetObj = $session->BuildSQLQuery($queryStr);
    $resultSetObj->Execute();
    while ($resultSetObj->MoveNext() == $CQPerlExt::CQ_SUCCESS) {
       $defaultTableDisplayName = $resultSetObj->GetColumnValue(1);
       $defaultFieldDisplayName = $resultSetObj->GetColumnValue(2);
       $cqFieldName = $resultSetObj->GetColumnValue(3);
       $displayName = $defaultTableDisplayName . "." . $defaultFieldDisplayName;
       $cqFieldName =~ tr/A-Z/a-z/;  # CQ lowercases all names
       printf "%-50s %-25s\n", $displayName, $cqFieldName;
       write (OUTPUTFILE);
    }

    #foreach $key (keys %pjcFieldHash)
    #{
    #   #print "$key		$pjcFieldHash{$key}\n";
    #   printf "%-50s %-25s\n", $key, $pjcFieldHash{$key};
    #   $physicalname = $pjcFieldHash{$key};
    #   write (OUTPUTFILE);
    #}

    close (OUTPUTFILE);

    print "\nOutput also saved in file CQFieldNames.txt.\n\n";
}

sub loadPjCTableNames{
    if ($hashTablesLoaded == BOOLEAN_TRUE)
    {
       return;
    }

    #print "Get PjC target table names.  Please wait...\n";

    #
    # Get table name result set (from dbm_table)
    #
    my $pjcTableNameResultSet = getPjCTableNames();
    #
    # Loop through each table name and print out
    #
    #print "process each row...\n";
    $count = 0;
    while ($pjcTableNameResultSet->MoveNext() == $CQPerlExt::CQ_SUCCESS) {
	$pjcTableName = $pjcTableNameResultSet->GetColumnValue(1);
        $pjcTableDbName = $pjcTableNameResultSet->GetColumnValue(2);
        #print "PjC dbm_table: Name = $pjcTableName, Db_Name = $pjcTableDbName\n";
        $pjcTableHash{$pjcTableName} = $pjcTableDbName;

        $count = $count + 1;

    }

#debug
#    print "$count table names in dbm_table" . "\n";
#    foreach $key (keys %pjcTableHash)
#    {
#       print "at $key we have $pjcTableHash{$key}\n";
#    }
}

sub getPjCTableNames {
    #
    # Return a result set containing the PjC table names
    # using raw SQL command
    #

    #print "getTableNames started.\n";
    $queryStr = "SELECT  default_display_name_1, name" .
                " FROM dbm_table";
    #print "Excute query $queryStr...\n";

    my $projResultSet = $session->BuildSQLQuery($queryStr);
    $projResultSet->Execute();

    #print "getTableNames completed.\n";
    return $projResultSet;
}


sub loadEntityDefTableNames {
    if ($hashTablesLoaded == BOOLEAN_TRUE) 
    {
       return;
    }

    #print "Get entitydef table names.  Please wait...\n";

    #
    # Get table name result set (from entitydef)
    #
    my $entitydefNameResultSet = getTableNames();
    #
    # Loop through each table name and print out
    #
    #print "process each row...\n";
    $count = 0;
    while ($entitydefNameResultSet->MoveNext() == $CQPerlExt::CQ_SUCCESS) {
	$entitydefTableName = $entitydefNameResultSet->GetColumnValue(1);
        $entitydefTableDbName = $entitydefNameResultSet->GetColumnValue(2);
        #print "Entitydef: Name = $entitydefTableName, Db_Name = $entitydefTableDbName\n";
        $entityDefHash{$entitydefTableName} = $entitydefTableDbName;

        $count = $count + 1;

    }

# debug
#    print "$count table names in entitydef" . "\n";
#    foreach $key (keys %entityDefHash)
#    {
#       print "at $key we have $entityDefHash{$key}\n";
#    }
}

sub getTableNames {
    #
    # Return a result set containing the CQ table names
    # using raw SQL command
    #

    #print "getTableNames started.\n";
    $queryStr = "SELECT  name, db_name " .
                " FROM entitydef";
    #print "Excute query $queryStr...\n";

    my $projResultSet = $session->BuildSQLQuery($queryStr);
    $projResultSet->Execute();

    #print "getTableNames completed.\n";
    return $projResultSet;
}

sub loadFieldDefNames {
    if ($hashTablesLoaded == BOOLEAN_TRUE) 
    {
       return;
    }

    #print "Get fielddef field names.  Please wait...\n";

    #
    # Get field name result set (from fielddef)
    #
    my $fielddefNameResultSet = getFieldDefNames();
    #
    # Loop through each table name and print out
    #
    #print "process each row...\n";
    #print "   EntityDef.name, EntityDef.db_name, " .
    #          "FieldDef.name, FieldDef.db_Name\n";
    #print "   ------------------------------------------------------------";
    $count = 0;
    while ($fielddefNameResultSet->MoveNext() == $CQPerlExt::CQ_SUCCESS) {
	$entitydefName = $fielddefNameResultSet->GetColumnValue(1);
        $entitydefDbName = $fielddefNameResultSet->GetColumnValue(2);
	$fielddefName = $fielddefNameResultSet->GetColumnValue(3);
        $fielddefDbName = $fielddefNameResultSet->GetColumnValue(4);
        #print "   $entitydefName, $entitydefDbName, $fielddefName, $fielddefDbName\n";
        #$fieldDefHash{$entitydefName . $fielddefName} = $fielddefDbName;
        $fieldDefHash{$entitydefDbName . "." . $fielddefDbName} = $fielddefDbName;

        $count = $count + 1;

    }

# debug
#format Z1 =
#@<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< @<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
#$key,                                        $z1
#.
#    open (Z1, ">z1.txt");
#    print "=========================================\n";
#    print "DEBUG: Dump field names in fielddef\n";
#    foreach $key (keys %fieldDefHash)
#    {
#       $z1 = $fieldDefHash{$key};
#       print "PHY FIELDS: At $key we have $fieldDefHash{$key}\n";
#       write (Z1);
#    }
#    print "=========================================\n";
}

sub getFieldDefNames {
    #
    # Return a result set containing the CQ field names
    # using raw SQL command
    #

    #print "getFieldDefNames started.\n";
    $queryStr = "SELECT  entitydef.name, entitydef.db_name, fielddef.name, fielddef.db_name" .
                " FROM entitydef, fielddef" . 
                " WHERE (entitydef.id = fielddef.entitydef_id)";
    #print "Excute query $queryStr...\n";

    my $projResultSet = $session->BuildSQLQuery($queryStr);
    $projResultSet->Execute();

    #print "getFieldDefNames completed.\n";
    return $projResultSet;
}

sub loadPjCFieldNames {
    if ($hashTablesLoaded == BOOLEAN_TRUE) 
    {
       return;
    }

    #print "Get PjC field names.  Please wait...\n";

    #
    # Get field name result set (from dbm_field)
    #
    my $fieldNameResultSet = getPjcFieldNames();
    #
    # Loop through each table name and print out
    #
    #print "process each row...\n";
    #print "   dbm_table.name, dbm_table.db_name, " .
    #          "dbm_field.name, dbm_field.db_Name\n";
    #print "   ------------------------------------------------------------";
    $count = 0;
    while ($fieldNameResultSet->MoveNext() == $CQPerlExt::CQ_SUCCESS) {
	$pjcTableName = $fieldNameResultSet->GetColumnValue(1);
        $pjcTableDbName = $fieldNameResultSet->GetColumnValue(2);
	$pjcFieldName = $fieldNameResultSet->GetColumnValue(3);
        $pjcFieldDbName = $fieldNameResultSet->GetColumnValue(4);
        #print "$pjcTableName, $pjcTableDbName, $pjcFieldName, $pjcFieldDbName\n";
        ###$pjcFieldHash{$pjcTableName . $pjcFieldName} = $pjcTableDbName . $pjcFieldDbName;

        # All names in CQ are lowercased, so lowercase the values
        $temp = $pjcTableDbName . "." . $pjcFieldDbName;
        $temp =~ tr/A-Z/a-z/; # lowercase everything
        $pjcFieldHashKey{$pjcTableName . "." . $pjcFieldName} = $temp;

        # All names in CQ are lowercased, so lowercase the values
        $temp = $pjcFieldDbName;
        $temp =~ tr/A-Z/a-z/; # lowercase everything
        $pjcFieldHash{$pjcTableName . "." . $pjcFieldName} = $temp;

        $count = $count + 1;

    }

# debug
#format Z2 =
#@<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< @<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
#$key,                                        $z2
#.
#    open (Z2, ">z2.txt");
#    print "=========================================\n";
#    print "$count field names in dbm_field" . "\n";
#    foreach $key (keys %pjcFieldHashKey)
#    {
#       $z2 = $pjcFieldHashKey{$key};
#       print "CQ Field: At $key we have $pjcFieldHashKey{$key}\n";
#       write (Z2);
#    }
#    print "=========================================\n";
}

sub getPjcFieldNames {
    #
    # Return a result set containing the PjC field names
    # using raw SQL command
    #

    #print "getPjcFieldNames started.\n";
    $queryStr = "SELECT  dbm_table.default_display_name_1, dbm_table.name," .
                "dbm_field.default_display_name_1, dbm_field.name " .
                " FROM dbm_table, dbm_field" . 
                " WHERE (dbm_table.dbid = dbm_field.table_fk)";
    #print "Excute query $queryStr...\n";

    my $projResultSet = $session->BuildSQLQuery($queryStr);
    $projResultSet->Execute();

    #print "getPjcFieldNames completed.\n";
    return $projResultSet;
}

sub setDimTableValue {
    my ($dimTableName, 
        $dimTableFieldName,
        $dimTableFieldValue
        ) = @_;

    $entity = $session->BuildEntity($dimTableName);
    $failure = $entity->SetFieldValue($dimTableFieldName, $dimTableFieldValue);
    $retVal = $entity->Validate();
    printf("entity->Validate() return value = $retVal\n");
    $entity->Commit();
    printf("entity->Commit() return value = $retVal\n");
    
}
sub getPhysicalReferenceValue {
}

sub getPhysicalReferenceDBID {
    my ($dimTableName, 
        $dimTableFieldName,
        $dimTableFieldValue
        ) = @_;

    $cqTableName = $pjcTableHash{$dimTableName};
print "\$cqTableName = $cqTableName\n";
    $cqFieldName = $pjcFieldHash{$dimTableName . "." . $dimTableFieldName};
print "\$cqFieldName = $cqFieldName\n";
    #my $queryDefObj = $session->BuildQuery($cqTableName);
    
    #$queryDefObj->BuildField("dbid");
     
    #$operator = $queryDefObj->BuildFilterOperator($CQPerlExt::AD_BOOL_OP_AND);

    #$operator->BuildFilter($cqFieldName, $CQPerlExt::AD_COMP_OP_EQ, $dimTableFieldValue);

    #$resultSetObj = $session->BuildResultSet($queryDefObj);
    #$resultSetObj->Execute();


    $queryStr = "SELECT dbid FROM $cqTableName " . 
                " WHERE ($cqFieldName = $dimTableFieldValue)";
    print "Excute query $queryStr...\n";

    my $projResultSet = $session->BuildSQLQuery($queryStr);
    $projResultSet->Execute();

    while ($projResultSet->MoveNext() == $CQPerlExt::CQ_SUCCESS) {
       $result = $projResultSet->GetColumnValue(1);
        print "    dbid = $result\n";
    }
    return $result;
}

1;    


